from docutils.core import publish_parts as rest
installed_markups = set(['rest'])
try:
    from markdown import markdown
    installed_markups.add('markdown')
except ImportError:
    pass
try:
    from textile import textile
    installed_markups.add('textile')
except ImportError:
    pass
try:
    import creole
    import creole2html
    installed_markups.add('creole')
except ImportError:
    pass

from swsg.loggers import swsg_logger

SUPPORTED_MARKUP_LANGUAGES = frozenset(
    ['rest', 'creole', 'textile', 'markdown'])


class UnsupportedMarkup(Exception):
    def __init__(self, markup):
        self.markup = markup

    def __str__(self):
        return 'the markup {0} is not supported'.format(self.markup)

    def __repr__(self):
        return '{0}({1})'.format(self.__class__.__name__, self.markup)


class BaseSource(object):
    def __init__(self, text):
        self.text = text
        first_line, sep, rest = text.partition('\n')
        if first_line.startswith('title:'):
            self.title = first_line.lstrip('title:').strip()
        else:
            # if the title is not given, generate it by taking the first five
            # words of the first line
            splitted_first_line = first_line.split()
            first_five_words = splitted_first_line[:5]
            self.title = ' '.join(first_five_words)
            # add a faked ellipsis to the title if there are more than 5 words
            # in the first line, i.e. if it was truncated
            if len(first_five_words) > 5:
                self.title += ' ...'

    def __eq__(self, other):
        return type(self) == type(other) and self.text == other.text

    def __ne__(self, other):
        return not (self == other)

    def __hash__(self):
        return hash(self.text)


class ReSTSource(BaseSource):
    def render(self):
        return rest(self.text, writer_name='html')['body']


class CreoleSource(BaseSource):
    def render(self):
        return creole2html.HtmlEmitter(creole.Parser(self.text).parse()).emit()


class TextileSource(BaseSource):
    def render(self):
        # the function textile.textile adds a tab character at the start of the
        # output, so we remove it to normalize the return value
        return textile(self.text).lstrip('\t')


class MarkdownSource(BaseSource):
    def render(self):
        return markdown(self.text, output_format='xhtml')


def get_source_class_by_markup(markup):
    markups = {
        'rest': ReSTSource,
        'creole': CreoleSource,
        'textile': TextileSource,
        'markdown': MarkdownSource}
    try:
        SourceClass = markups[markup]
    except KeyError:
        raise UnsupportedMarkup(markup)
    else:
        return SourceClass


def ensure_markup_is_valid_and_installed(markup_language, logger=swsg_logger):
    if markup_language not in SUPPORTED_MARKUP_LANGUAGES:
        raise ValueError(
            '{0} is not an allowed value for "markup_language"; '
            'possible valid values are: {1}'.format(
                markup_language, SUPPORTED_MARKUP_LANGUAGES))
    elif markup_language not in installed_markups:
        if markup_language == 'rest':
            missing_package = 'docutils'
        else:
            missing_package = markup_language
        logger.critical('The package "{0}" is not installed.'.format(
            missing_package))



from os import getenv, path

XDG_DATA_HOME = getenv(
    'XDG_DATA_HOME', path.expanduser(path.join('~', '.local', 'share')))
PROJECT_DATA_DIR = path.join(XDG_DATA_HOME, 'swsg')
LOGFILE = path.join(PROJECT_DATA_DIR, 'swsg.log')
DEFAULT_PROJECTS_FILE_NAME = path.join(PROJECT_DATA_DIR, 'projects.shelve')



import os
import shutil
import shelve
import contextlib
from datetime import datetime
from itertools import izip
from ConfigParser import SafeConfigParser, DuplicateSectionError

from swsg.loggers import swsg_logger as logger
from swsg.file_paths import DEFAULT_PROJECTS_FILE_NAME
from swsg.templates import SimpleTemplate
from swsg.sources import get_source_class_by_markup


class NonexistingProject(Exception):
    pass


class Project(object):
    CONFIG_SECTION = 'local configuration'

    def __init__(self, path, name,
                 projects_file_name=DEFAULT_PROJECTS_FILE_NAME):
        self.path = os.path.abspath(path)
        self.name = name
        self.config = SafeConfigParser()

        # there is no empty value for datetime.datetime, so ``None`` is used
        self.created = None
        self.last_modified = None

        # the following values can currently only be changed by subclassing
        # this class
        self.project_dir = os.path.join(self.path, self.name)
        self.source_dir = os.path.join(self.project_dir, 'sources')
        self.template_dir = os.path.join(self.project_dir, 'templates')
        self.output_dir = os.path.join(self.project_dir, 'output')
        self.config_filename = os.path.join(self.project_dir, 'config.ini')
        self.projects_file_name = projects_file_name

        # True after the projects file was updated
        self.updated_projects_file = False

    def __repr__(self):
        return '{0} "{1}"'.format(self.__class__.__name__, self.name)

    def init(self):
        '''create the specific project directory and add a configuration file
        with default values

        '''
        self.make_project_directories()
        self.reset_config()
        self.read_config()
        self.update_projects_file(new_created=True)

    def make_project_directories(self):
        logger.notice('creating project directories')
        for path_name in (self.source_dir, self.template_dir, self.output_dir):
            logger.info('creating the directory {0}'.format(path_name))
            os.makedirs(path_name)

    @property
    def exists(self):
        dir_paths = (
            self.project_dir, self.source_dir,
            self.template_dir, self.output_dir
        )
        file_paths = (self.config_filename, self.projects_file_name)
        paths = dir_paths + file_paths
        return (
            all(os.path.exists(path) for path in paths) and
            all(os.path.isdir(dir_path) for dir_path in dir_paths) and
            all(os.path.isfile(file_path) for file_path in file_paths) and
            self.updated_projects_file
        )

    @property
    def sources(self):
        for source_name in os.listdir(self.source_dir):
            # the markup language is the filename extension without the dot.
            # For example, the content of "foo.rest" will be rendered as ReST
            markup_language = os.path.splitext(source_name)[1].lstrip('.')
            source_path = os.path.join(self.source_dir, source_name)
            with open(source_path) as fp:
                text = fp.read().decode('utf-8')
            SourceClass = get_source_class_by_markup(markup_language)
            yield SourceClass(text)

    @property
    def templates(self):
        self.read_config()
        template_language = self.config.get(
            self.CONFIG_SECTION, 'template language')
        # FIXME: add more template classes
        templates = {
            'simple': SimpleTemplate}
        try:
            TemplateClass = templates[template_language]
        except KeyError:
            logger.critical(
                'the template language {0} does either '
                'not exist or is not supported.')
        for template_name in os.listdir(self.template_dir):
            filename = os.path.join(self.template_dir, template_name)
            with open(filename) as fp:
                file_content = fp.read().decode('utf-8')
            yield TemplateClass(file_content), filename

    def update_projects_file(self, new_created=False):
        now = datetime.now()
        if new_created:
            logger.notice(
                'creating the projects file {0}'.format(
                    self.projects_file_name))
            self.created = now
        self.last_modified = now
        with contextlib.closing(shelve.open(self.projects_file_name)) as p:
            logger.notice(
                'updating the projects file {0}'.format(
                    self.projects_file_name))
            p[self.project_dir] = self
        self.updated_projects_file = True

    def read_config(self):
        logger.notice('reading the configuration file')
        with open(self.config_filename) as fp:
            self.config.readfp(fp)

    def reset_config(self):
        logger.notice('resetting the configuration file')
        options = ['template language']
        default_values = ['simple']
        default_settings = izip(options, default_values)
        try:
            self.config.add_section(self.CONFIG_SECTION)
        except DuplicateSectionError:
            # the sectiion does already exist, so it will be removed including
            # all its entries before adding it as a new section
            self.config.remove_section(self.CONFIG_SECTION)
            for option in options:
                self.config.remove_option(self.CONFIG_SECTION, option)
            self.config.add_section(self.CONFIG_SECTION)
        for option, value in default_settings:
            self.config.set(self.CONFIG_SECTION, option, value)
        with open(os.path.join(self.project_dir, 'config.ini'), 'w') as fp:
            self.config.write(fp)

    def update_config(self, markup_language=None, template_language=None):
        '''Set the values for "markup language" and "template language" to
        ``markup_language`` or ``template_language``, respectively. If one of
        the arguments is None (the default), its corresponding value in the
        configuration file won't change.

        '''
        self.read_config()
        options = ('markup language', 'template language')
        values = (markup_language, template_language)
        for option, value in izip(options, values):
            if value is not None:
                logger.info('setting the option {0} to the value {1}'.format(
                    option, value))
                self.config.set(self.CONFIG_SECTION, option, value)
        with open(self.config_filename, 'w') as fp:
            self.config.write(fp)
        self.updated_projects_file = True

    def render(self):
        logger.notice('starting the rendering process')
        for template, template_filename in self.templates:
            try:
                for source_name, output in template.render(self.source_dir):
                    head, tail = os.path.split(source_name)
                    filename = os.path.splitext(tail)[0]
                    output_path = os.path.join(
                        self.output_dir, filename) + '.html'
                    logger.info('{0} + {1} -> {2}'.format(
                        source_name, template_filename, output_path))
                    yield output_path, output
            except NonexistingSource, e:
                logger.critical(str(e))
        logger.notice('finishing the rendering process')

    def save_source(self, source, name):
        logger.notice('saving the source {0} in the directory {1}'.format(
            name, self.source_dir))
        filename = os.path.join(self.source_dir, name)
        with open(filename, 'w') as fp:
            fp.write(source.text)
        self.update_projects_file()

    def save_template(self, template, name):
        logger.notice('saving the template {0} in the directory {1}'.format(
            name, self.template_dir))
        filename = os.path.join(self.template_dir, name)
        with open(filename, 'w') as fp:
            fp.write(template.text)
        self.update_projects_file()


def list_project_instances(projects_file_name=DEFAULT_PROJECTS_FILE_NAME):
    'get all ``Project`` instances which can be found in the projects file'
    with contextlib.closing(shelve.open(projects_file_name)) as projects:
        return projects.values()


def remove_project(project):
    '''remove both the project's directory and its entry in the projects file

    '''
    proj_dir = project.project_dir
    if project.exists:
        shutil.rmtree(proj_dir)
        with contextlib.closing(shelve.open(project.projects_file_name)) as p:
            p.pop(proj_dir)
    else:
        # project does not exist, therefore it cannot be removed
        raise NonexistingProject(
            'The project {0} with its belonging '
            'directory {1} does not exist.'.format(project, proj_dir))



#!/usr/bin/env python

from __future__ import print_function

import sys
import codecs
from os import path, name as operating_system
from itertools import imap

from argparse import ArgumentParser
from texttable import Texttable
from py.io import TerminalWriter
from logbook import FileHandler, INFO, DEBUG, ERROR
from progressbar import ProgressBar

from swsg import __version__
from swsg.loggers import swsg_logger as logger
from swsg.file_paths import LOGFILE as DEFAULT_LOGFILE
from swsg.projects import Project, list_project_instances, remove_project
from swsg.sources import SUPPORTED_MARKUP_LANGUAGES
from swsg.templates import SUPPORTED_TEMPLATE_ENGINES
from swsg.utils import is_none


def get_logging_handler(args, logging_level=ERROR):
    if args.logfile is None:
        if operating_system == 'nt':
            # FIXME: use the handler logbook.NTEventLogHandler for windows
            #        users
            raise OSError(
                'Windows is an operating system which is not supported yet. '
                'Install a POSIX compatible system and try again.')
        logfile = DEFAULT_LOGFILE
    else:
        logfile = args.logfile
    return FileHandler(logfile, level=logging_level)


def format_list_of_projects():
    terminal_writer = TerminalWriter()
    terminal_width = terminal_writer.fullwidth
    table = Texttable(max_width=terminal_width)
    table.header(['Name', 'Path', 'Created', 'Last modified'])
    projects = list_project_instances()
    for p in projects:
        table.add_row([
            p.name, p.path,
            p.created.strftime('%c'), p.last_modified.strftime('%c')])
    return table.draw() if projects else "no project created yet"


def print_list_of_projects(args):
    print(format_list_of_projects())


def init_project(args):
    project = Project(args.project_directory, args.name)
    project.init()


def remove_project_(args):
    project = Project(args.project_directory, args.path)
    remove_project(project)
    print('removed the project {0}'.format(project.project_dir))


def validate_change_config(args):
    # at least one of the options must be given (otherwise, the file is only
    # touched but not edited)
    conf_values = [args.markup_language, args.template_language]
    if all(imap(is_none, conf_values)):
        print(
            'Error: Neither a markup language '
            'nor a template language was given.',
            file=sys.stderr)
        # the exit code 2 is used for CLI errors by convention
        sys.exit(2)


def change_config(args):
    # the project's directory is the current working directory
    project = Project(*path.split(path.abspath(path.curdir)))
    project.update_config(
        markup_language=args.markup_language,
        template_language=args.template_language)


def render(args):
    pbar = ProgressBar(maxval=100)
    # the project's directory is the current working directory
    project = Project(*path.split(path.abspath(path.curdir)))
    for i, (output_path, output) in enumerate(project.render()):
        with codecs.open(output_path, 'w', 'utf-8') as fp:
            fp.write(output)
        pbar.update(i + 1)


def parse_args(argv=sys.argv[1:]):
    # TODO: improve the help by using the method ``parser.add_argument_group``
    #       which groups the help message by subarguments
    parser = ArgumentParser()
    parser.add_argument(
        '-v', '--verbose', action='store_true',
        help=(
            'Enable verbosity mode (disabled per default). Log more logging '
            'messages as usual, but no debugging messages.'))
    parser.add_argument(
        '-d', '--debug', action='store_true',
        help=(
            'Enable debugging mode (disabled per default). This option causes '
            'SWSG to log as many messages as possible and therefore overrides '
            'the option "verbose".'))
    parser.add_argument(
        '--version', action='version', version='%(prog)s ' + __version__)
    parser.add_argument('-l', '--logfile')
    subparsers = parser.add_subparsers()
    list_parser = subparsers.add_parser(
        'list-projects', help='List all projects in a fancy ASCII table.')
    list_parser.set_defaults(func=print_list_of_projects)
    init_parser = subparsers.add_parser(
        'init', help='create and initialize a new project')
    init_parser.add_argument('name', help='The name of the project')
    init_parser.add_argument(
        '-p', '--project-directory', default='.',
        help=(
            'The directory where the new project will be created. It must '
            'already exist before the calling this command.'))
    init_parser.set_defaults(func=init_project)
    remove_parser = subparsers.add_parser(
        'remove-project', help='remove a project which does already exist')
    remove_parser.add_argument(
        'path',
        help='The path to the project directory which will be removed')
    remove_parser.set_defaults(func=remove_project)
    config_parser = subparsers.add_parser(
        'change-config',
        help=(
            'Change project-dependent configuration values. It is recommended '
            'to use this interface instead of editing the configuration file '
            'config.ini directly.'))
    config_parser.add_argument(
        '-m', '--markup-language', choices=SUPPORTED_MARKUP_LANGUAGES,
        help=(
            'The name of the markup language being used. Possible valid values'
            ' are: {0}'.format(', '.join(SUPPORTED_MARKUP_LANGUAGES))))
    config_parser.add_argument(
        '-t', '--template-language', choices=SUPPORTED_TEMPLATE_ENGINES,
        help=(
            'The name of the template engine being used. Possible valid values'
            ' are: {0}'.format(', '.join(SUPPORTED_TEMPLATE_ENGINES))))
    config_parser.set_defaults(func=change_config)
    render_parser = subparsers.add_parser(
        'render',
        help=(
            'Render the templates with their corresponding source files which '
            'are located in the current project directory.'))
    render_parser.set_defaults(func=render)
    return parser.parse_args(argv)


def set_logging_level(args, logger):
    if args.verbose:
        logger.level_name = INFO
    if args.debug:
        logger.level_name = DEBUG
    return logger


def main(argv=sys.argv[1:]):
    args = parse_args(argv)
    if args.func == change_config:
        validate_change_config(args)
    level = set_logging_level(args, logger)
    handler = get_logging_handler(args, level)
    with handler.applicationbound():
        args.func(args)

if __name__ == '__main__':
    main()



'''
About
=====
swsg (static web site generator) is a tool to create static websites using
human-readable markup languages like ReST_. It was highly-inspired by SR_.
The advantage of SWSG against directly using (X)HTML files is that you have one
(or more) templates and do not have to rewrite every your main layout on ebery
single site. You can concentrate on the main content: the text; the content is
seperated from the templates.

Features
========
- supports many markup languages: Markdown_, Creole_, Textile_, ReST_
- supports many template engines: Jinja2_, Mako_, Genshi_, and also a very
  simple template language for those who do not need control structures like
  loops or if-conditions -> only the simple template language is supported yet.
- supports clevercss_ beside the usual CSS as a markup language for the
  stylesheets -> not yet!
- provides multiple interfaces: until now, only a CLI is implemented, but
  others are planned:

  - PIDA_-Plugin
  - web interface
  - possibly a GTK+ or Qt application, but I think using the PIDA-Plugin is
    more comfortable

Requirements
============
- either Python 2.7 or Python 2.6 with the python package argparse_ installed
- texttable_
- `py`_
- `progressbar`_
- at least one of the following markup languages:

  - ReST_
  - markdown_
  - creole_
  - textile_
- optional: one or more of the following template engines, also installed as
  python packages:

  - `jinja2`_
  - `mako`_
  - `genshi`_

.. _ReST: http://docutils.sourceforge.net/rst.html
.. _SR: http://bitbucket.org/tiax/sr/overview
.. _markdown: http://daringfireball.net/projects/markdown/
.. _creole: http://www.wikicreole.org/
.. _textile: http://textile.thresholdstate.com/
.. _jinja2: http://jinja.pocoo.org/2/
.. _mako: http://www.makotemplates.org/
.. _Genshi: http://genshi.edgewall.org/
.. _clevercss: http://sandbox.pocoo.org/clevercss/
.. _PIDA: http://pida.co.uk/
.. _argparse: http://code.google.com/p/argparse/
.. _texttable: http://pypi.python.org/pypi/texttable
.. _py: http://pypi.python.org/pypi/py
.. _progressbar: http://code.google.com/p/python-progressbar/
'''

from __future__ import print_function

from setuptools.command.install import install
from setuptools import setup

import os

from swsg import __version__
from swsg.file_paths import LOGFILE as DEFAULT_LOGFILE, PROJECT_DATA_DIR


class LogfileAndDirectoryInitializer(install):
    '''create the directories where the projects file and the logging file will
    be saved if they do not exist yet and create an empty logfile'''
    def initialize_dirs(self):
        path = PROJECT_DATA_DIR
        if not os.path.exists(path):
            print('creating the directory {0}'.format(path))
            os.makedirs(path)

    def create_empty_logfile(self):
        with open(DEFAULT_LOGFILE, 'w') as f:
            print('creating the logfile {0}'.format(f.name))

    def run(self):
        self.initialize_dirs()
        self.create_empty_logfile()
        install.run(self)

short_description = (
    'SWSG (Static Web Site Generator) is a tool to generate static web pages.')

setup(
    name='swsg',
    description=short_description,
    long_description=__doc__,
    version=__version__,
    author='Simon Liedtke',
    author_email='liedtke.simon@googlemail.com',
    url='http://github.com/derdon/swsg',
    license='WTFPL',
    packages=['swsg'],
    install_requires=['docutils', 'py', 'texttable', 'progressbar'],
    extras_require={
        'markdown': ['markdown'],
        'textile': ['textile'],
        'creole': ['creole'],
        'mako': ['mako'],
        'jinja2': ['jinja2']},
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        #'Environment :: Console :: Curses',
        #'Environment :: Plugins',
        #'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: POSIX',
        'Operating System :: Unix',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Text Processing',
        'Topic :: Text Processing :: Markup'],
    entry_points={
        'console_scripts': [
            'swsg-cli = swsg.cli:main',
         ],
    },
    cmdclass={
        'install': LogfileAndDirectoryInitializer
    },
)
